=== modified file 'gce.py'
--- gce.py	2016-11-21 19:53:14 +0000
+++ gce.py	2016-11-29 02:02:19 +0000
@@ -56,12 +56,15 @@
     return young
 
 
-def get_client(sa_email, pem_path, project_id):
+def get_client(sa_email, pem_path, project_id, region=None):
     """Delay imports and activation of GCE client as needed."""
     import libcloud
     gce = libcloud.compute.providers.get_driver(
         libcloud.compute.types.Provider.GCE)
-    return gce(sa_email, pem_path, project=project_id)
+    client = gce(sa_email, pem_path, project=project_id, datacenter=region)
+    if region and client.ex_get_zone(region) is None:
+        raise ValueError("Unknown region: ", region)
+    return client
 
 
 def list_instances(client, glob='*', print_out=False):
@@ -75,7 +78,7 @@
     :return: A list of Nodes
     """
     nodes = []
-    for node in client.list_nodes(ex_zone='all'):
+    for node in client.list_nodes():
         if not fnmatch.fnmatch(node.name, glob):
             log.debug('Skipping {}'.format(node.name))
             continue
@@ -155,6 +158,7 @@
         help=("The secret to make requests with. "
               "Environment: $GCE_PROJECT_ID."),
         default=os.environ.get('GCE_PROJECT_ID'))
+    parser.add_argument('--region', help="The compute engine region.")
     subparsers = parser.add_subparsers(help='sub-command help', dest="command")
     ls_parser = subparsers.add_parser(
         'list-instances', help='List vm instances.')
@@ -181,7 +185,8 @@
 def main(argv):
     args = parse_args(argv)
     log.setLevel(args.verbose)
-    client = get_client(args.sa_email, args.pem_path, args.project_id)
+    client = get_client(args.sa_email, args.pem_path, args.project_id,
+                        region=args.region)
     try:
         if args.command == 'list-instances':
             list_instances(client, glob=args.filter, print_out=True)

=== modified file 'tests/test_gce.py'
--- tests/test_gce.py	2016-11-21 19:53:14 +0000
+++ tests/test_gce.py	2016-11-29 02:02:19 +0000
@@ -60,7 +60,23 @@
         self.assertEqual(0, code)
         gc_mock.assert_called_once_with(
             GCE_ENVIRON['GCE_SA_EMAIL'], GCE_ENVIRON['GCE_PEM_PATH'],
-            GCE_ENVIRON['GCE_PROJECT_ID'])
+            GCE_ENVIRON['GCE_PROJECT_ID'], region=None)
+        li_mock.assert_called_once_with(
+            client, glob='juju-deploy*', print_out=True)
+
+    def test_main_list_instances_region(self):
+        client = make_fake_client()
+        with patch('gce.get_client', autospec=True,
+                   return_value=client) as gc_mock:
+            with patch('gce.list_instances', autospec=True) as li_mock:
+                with patch.dict('os.environ', GCE_ENVIRON):
+                    code = gce.main(
+                        ['gce.py', '--region', 'test-region', 'list-instances',
+                         'juju-deploy*'])
+        self.assertEqual(0, code)
+        gc_mock.assert_called_once_with(
+            GCE_ENVIRON['GCE_SA_EMAIL'], GCE_ENVIRON['GCE_PEM_PATH'],
+            GCE_ENVIRON['GCE_PROJECT_ID'], region='test-region')
         li_mock.assert_called_once_with(
             client, glob='juju-deploy*', print_out=True)
 
@@ -76,10 +92,46 @@
         self.assertEqual(0, code)
         gc_mock.assert_called_once_with(
             GCE_ENVIRON['GCE_SA_EMAIL'], GCE_ENVIRON['GCE_PEM_PATH'],
-            GCE_ENVIRON['GCE_PROJECT_ID'])
+            GCE_ENVIRON['GCE_PROJECT_ID'], region=None)
         di_mock.assert_called_once_with(
             client, 'juju-deploy*', old_age=2, dry_run=False)
 
+    def test_get_client(self):
+        client = Mock(spec=['ex_get_zone'])
+        client.return_value.ex_get_zone.return_value = 'bar'
+        with patch('libcloud.compute.providers.get_driver', autospec=True
+                   ) as gl_mock:
+            gl_mock.return_value = client
+            gce.get_client(GCE_ENVIRON['GCE_SA_EMAIL'],
+                       GCE_ENVIRON['GCE_PEM_PATH'],
+                       GCE_ENVIRON["GCE_PROJECT_ID"], region='foo')
+        gl_mock.assert_called_once_with('gce')
+        client.return_value.ex_get_zone.assert_called_once_with('foo')
+
+    def test_get_client_no_region(self):
+        client = Mock(spec=['ex_get_zone'])
+        client.return_value.ex_get_zone.return_value = 'bar'
+        with patch('libcloud.compute.providers.get_driver', autospec=True
+                   ) as gl_mock:
+            gl_mock.return_value = client
+            gce.get_client(GCE_ENVIRON['GCE_SA_EMAIL'],
+                       GCE_ENVIRON['GCE_PEM_PATH'],
+                       GCE_ENVIRON["GCE_PROJECT_ID"], region=None)
+        gl_mock.assert_called_once_with('gce')
+        self.assertEqual(client.return_value.ex_get_zone.call_count, 0)
+
+    def test_get_client_exception(self):
+        client = Mock(spec=['ex_get_zone'])
+        client.return_value.ex_get_zone.return_value = None
+        with patch('libcloud.compute.providers.get_driver', autospec=True
+                   ) as gl_mock:
+            gl_mock.return_value = client
+            with self.assertRaises(ValueError):
+                gce.get_client(GCE_ENVIRON['GCE_SA_EMAIL'],
+                           GCE_ENVIRON['GCE_PEM_PATH'],
+                           GCE_ENVIRON["GCE_PROJECT_ID"], region='foo')
+        gl_mock.assert_called_once_with('gce')
+
     def test_parse_args_delete_instaces(self):
         with patch.dict('os.environ', GCE_ENVIRON):
             args = gce.parse_args(
@@ -88,8 +140,8 @@
         expected = Namespace(
             command='delete-instances', dry_run=True, filter='juju-deploy*',
             old_age=2, pem_path='/gce-serveraccount.json',
-            project_id='test-project', sa_email='me@serviceaccount.google.com',
-            verbose=10)
+            project_id='test-project', region=None,
+            sa_email='me@serviceaccount.google.com', verbose=10)
         self.assertEqual(expected, args)
 
     def test_parse_args_list_instances(self):
@@ -98,8 +150,8 @@
                 ['gce.py', '-v', '-d', 'list-instances', 'juju-deploy*'])
         expected = Namespace(
             command='list-instances', dry_run=True, filter='juju-deploy*',
-            pem_path='/gce-serveraccount.json',
-            project_id='test-project', sa_email='me@serviceaccount.google.com',
+            pem_path='/gce-serveraccount.json', project_id='test-project',
+            region=None, sa_email='me@serviceaccount.google.com',
             verbose=10)
         self.assertEqual(expected, args)
 
@@ -113,8 +165,8 @@
         expected = Namespace(
             command='delete-instances', dry_run=True, filter='juju-deploy*',
             old_age=2, pem_path='/gce-serveraccount.json',
-            project_id='test-project', sa_email='me@serviceaccount.google.com',
-            verbose=10)
+            project_id='test-project', region=None,
+            sa_email='me@serviceaccount.google.com', verbose=10)
         self.assertEqual(expected, args)
 
     def test_is_permanent_true(self):
@@ -150,7 +202,7 @@
         client = make_fake_client()
         client.list_nodes.return_value = [no_node, yes_node]
         found = gce.list_instances(client, 'juju-*')
-        client.list_nodes.assert_called_once_with(ex_zone='all')
+        client.list_nodes.assert_called_once_with()
         self.assertEqual([yes_node], found)
 
     def test_list_instances_with_print(self):
@@ -162,7 +214,7 @@
         client.list_nodes.return_value = [node_one, node_two]
         with patch('sys.stdout', new_callable=StringIO) as so_sio:
             found = gce.list_instances(client, 'juju-*', print_out=True)
-        client.list_nodes.assert_called_once_with(ex_zone='all')
+        client.list_nodes.assert_called_once_with()
         self.assertEqual([node_one, node_two], found)
         self.assertEqual(
             'juju-controller\tUNKNOWN\tNone\trunning\njuju-app\tus-west1\t'
@@ -205,3 +257,4 @@
         self.assertEqual(
             'ERROR Cannot delete juju-old\n',
             self.log_stream.getvalue())
+

